/* 70-762 Lesson 12 Lab 3 - Implementing DMOs that Evaluate Index Usage */

/* EXAMPLE 1:
		Using sys.dm_db_index_usage_stats 
        to find indexes not used in resolving queries */

/*Relevant Columns: 
database_id  primary key of the database 
object_id - object_id of table index belongs to 
index_id - primary key of index 
user_seeks  number of times index used in user query in seek operation (one specific row) 
user_scans  number of times index used by scanning leaf pages of index for data 
user_lookups  for clustered indexes only, number of times index used in "bookmark lookup" to fetch full row 
               (non-clustered indexes use clustered indexes key as the pointer to base row) 
user_updates  number of times index has been modified due to change in the table's data 
last_user_seek  date and time of last user seek operation 
last_user_scan  date and time of last user scan operation 
last_user_lookup  date and time of the last user lookup operation 
last_user_update  date and time of the last user update operation 
*/

USE WideWorldImporters;
GO

SELECT object_schema_name(indexes.object_id) + '.' + object_name(indexes.object_id) AS 'Object Name',
indexes.name AS Name, 
		 CASE WHEN is_unique = 1
		     THEN 'UNIQUE ' 
			 ELSE '' end + indexes.type_desc AS 'Index Type', 
 ddius.user_seeks, 
		 ddius.user_scans, 
		 ddius.user_lookups, 
		 ddius.user_updates
FROM sys.indexes
LEFT OUTER JOIN sys.dm_db_index_usage_stats ddius
     ON indexes.object_id = ddius.object_id
  AND indexes.index_id = ddius.index_id
AND ddius.database_id = db_id()
ORDER BY ddius.user_seeks + ddius.user_scans + ddius.user_lookups DESC;
GO

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

/* EXAMPLE 2:
		Using sys.dm_db_index_physical_stats 
        to find index fragmentation */

 USE WideWorldImporters;
 GO
-->>> Average fragmentation in percent for WWI database <<<--
SELECT a.index_id, 
	   name,
	   avg_fragmentation_in_percent
FROM sys.dm_db_index_physical_stats (DB_ID(), OBJECT_ID(N'%%'),
NULL, NULL, NULL) AS a
JOIN sys.indexes AS b 
     ON a.object_id = b.object_id 
	 AND a.index_id = b.index_id
ORDER BY avg_fragmentation_in_percent DESC;
GO

/* EXAMPLE:Using sys.dm_db_index_physical_stats 
           to find index fragmentation in a table */
-->>>Average fragmentation of indexes in WideWorldImporters.Sales.InvoiceLines table <<<--
SELECT a.index_id, 
       name, 
	   avg_fragmentation_in_percent
FROM sys.dm_db_index_physical_stats (DB_ID(), OBJECT_ID(N'Sales.InvoiceLines'),
NULL, NULL, NULL) AS a
JOIN sys.indexes AS b 
     ON a.object_id = b.object_id 
	 AND a.index_id = b.index_id
ORDER BY avg_fragmentation_in_percent DESC;
GO

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

/* EXAMPLE 3:
	Using DMOs to find missing indexes
*/

USE WideWorldImporters;
GO

--Execute the query-------------------
SELECT [OrderID]
      ,[CustomerID]
      ,[SalespersonPersonID]
      ,[OrderDate]
  FROM Sales.Orders
  WHERE SalespersonPersonID = 7;
--Join DMOs to find missing indexes-----------------------------------
SELECT 
	(avg_user_impact*.01) *(user_seeks+user_scans)* avg_total_user_cost AS  Avg_Improvement_with_Index,
	mid.statement,
	mid.equality_columns,
	mid.inequality_columns,
	mid.included_columns
FROM sys.dm_db_missing_index_group_stats AS migs
	INNER JOIN sys.dm_db_missing_index_groups AS mig
		ON migs.group_handle = mig.index_group_handle
	INNER JOIN sys.dm_db_missing_index_details AS mid
		ON mig.index_handle = mid.index_handle
ORDER BY Avg_Improvement_with_Index DESC;
GO
