/*70-762 Lesson 2.8 Lab 4 - Analyzing Query Plans for Indexing Improvements */

--Begin Lab by reading a simple execution plan--------------------
--From slide------------------------------------------------------
SELECT CountryName, IsoNumericCode, Continent, Region
FROM Application.Countries
WHERE Subregion = 'Western Europe'
ORDER BY CountryName;

---------------------------------------------------------------------
--Using the Graphical Execution plan to display missing indexes
--and improve query performance--------------------------------------
USE WideWorldImporters
GO

SELECT CustomerID, OrderID, CustomerPurchaseOrderNumber, OrderDate, ExpectedDeliveryDate
FROM Sales.Orders 
WHERE CustomerPurchaseOrderNumber BETWEEN '12211' AND '12212';
GO
/*
Missing Index Details 
The Query Processor estimates that implementing the following index
could improve the query cost by 77.1437%.
*/

/*
USE [WideWorldImporters]
GO
CREATE NONCLUSTERED INDEX [<Name of Missing Index, sysname,>]
ON [Sales].[Orders] ([CustomerPurchaseOrderNumber])
INCLUDE ([OrderID],[CustomerID],[OrderDate],[ExpectedDeliveryDate])
GO
*/


--Using the Graphical Execution plan to display missing indexes
--and improve query performance--------------------------------------

Use WideWorldImporters;
GO

DROP TABLE IF EXISTS Examples.NewPurchaseOrders;
GO

DROP TABLE IF EXISTS Examples.NewPurchaseOrderLines;
GO

--Create two tables in Examples schema by copying two WWI tables---
--New tables have no Constraints and no Indexes--------------------
SELECT *
INTO Examples.NewPurchaseOrders
FROM Purchasing.PurchaseOrders;
GO



SELECT *
INTO Examples.NewPurchaseOrderLines
FROM Purchasing.PurchaseOrderLines;
GO


---------------------------------------------------------------------------------
SELECT *
FROM Examples.NewPurchaseOrders;
GO

SELECT PurchaseOrderID, ExpectedDeliveryDate
FROM Examples.NewPurchaseOrders
WHERE EXISTS (SELECT *
	FROM Examples.NewPurchaseOrderLines
	WHERE NewPurchaseOrderLines.PurchaseOrderID = NewPurchaseOrders.PurchaseOrderID
 AND OrderDate BETWEEN '2015-01-01' AND '2015-01-31');
GO

/*
Missing Index Details 
The Query Processor estimates that implementing the following index 
could improve the query cost by 88.3073%.
*/
/*
USE [WideWorldImporters]
GO
CREATE NONCLUSTERED INDEX [<Name of Missing Index, sysname,>]
ON [Examples].[NewPurchaseOrderLines] ([PurchaseOrderID]);
GO
*/



----------------------------------------------------------------------------------------
--Create PurchaseOrderID as PRIMARY KEY------
ALTER TABLE Examples.NewPurchaseOrders
	ADD CONSTRAINT PK_Examples_NewPurchasOrders PRIMARY KEY (PurchaseOrderID);
GO

--Create PurchaseOrderLineID as PRIMARY KEY------
ALTER TABLE Examples.NewPurchaseOrderLines
	ADD CONSTRAINT PK_Examples_NewPurchasOrderLines PRIMARY KEY (PurchaseOrderLineID);
GO

--Create FOREIGN KEY Constraint----
ALTER TABLE Examples.NewPurchaseOrderLines
	ADD CONSTRAINT FK_Examples_NewPurchaseOrderLines_PurchaseOrderID
	FOREIGN KEY (PurchaseOrderID) REFERENCES Examples.NewPurchaseOrders (PurchaseOrderID);
GO



-----------------------------------------------------------------------------------
-----------------------------------------------------------------------------------
--Using the Graphical Execution plan to display missing indexes
--and improve query performance--------------------------------------
Use WideWorldImporters;
GO

SELECT PO.ContactPersonID, POL.StockItemID, POL.Description, POL.ExpectedUnitPricePerOuter, POL.LastReceiptDate
FROM Purchasing.PurchaseOrderLines AS POL
JOIN Purchasing.PurchaseOrders AS PO
 ON POL.PurchaseOrderID = PO.PurchaseOrderID
WHERE POL.OrderedOuters  IN  (4, 9, 12,17) AND POL.ExpectedUnitPricePerOuter > 100.00
ORDER BY POL.ExpectedUnitPricePerOuter;

/*
Missing Index Details 
The Query Processor estimates that implementing the following index 
could improve the query cost by 50.8792%.
*/

/*
USE [WideWorldImporters]
GO
CREATE NONCLUSTERED INDEX [<Name of Missing Index, sysname,>]
ON [Purchasing].[PurchaseOrderLines] ([OrderedOuters],[ExpectedUnitPricePerOuter])
INCLUDE ([PurchaseOrderID],[StockItemID],[Description],[LastReceiptDate])
GO
*/



---------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------
--Extra sample using AdventureWorks2016CTP3-------------------------------------------
USE AdventureWorks2016CTP3 
GO

SELECT BusinessEntityID, Name, SalesPersonID, ModifiedDate 
FROM Sales.Store 
WHERE (Name='Bike World' AND ModifiedDate > '2004-10-01')
GO

SELECT PO.ContactPersonID, POL.StockItemID, POL.Description, POL.ExpectedUnitPricePerOuter, POL.LastReceiptDate
FROM Examples.NewPurchaseOrderLines AS POL
JOIN Examples.NewPurchaseOrders AS PO
 ON POL.PurchaseOrderID = PO.PurchaseOrderID
WHERE POL.OrderedOuters  IN  (4, 9, 12,17) AND POL.ExpectedUnitPricePerOuter > 100.00
ORDER BY POL.ExpectedUnitPricePerOuter;

/*
Missing Index Details 
The Query Processor estimates that implementing the following index
could improve the query cost by 48.2904%.
*/
/*
USE [WideWorldImporters]
GO
CREATE NONCLUSTERED INDEX [<Name of Missing Index, sysname,>]
ON [Examples].[NewPurchaseOrderLines] ([OrderedOuters],[ExpectedUnitPricePerOuter])
INCLUDE ([PurchaseOrderID],[StockItemID],[Description],[LastReceiptDate])
GO
*/
