/*70-762 Lesson 3.6 Lab 3 - Configuring and Using Partitioned Views */

Use WideWorldImporters;
GO

DROP TABLE IF EXISTS Examples.InvoicesPart_1;
GO

DROP TABLE IF EXISTS Examples.InvoicesPart_2;
GO

DROP TABLE IF EXISTS Examples.InvoicesPart_3;
GO

--Create and Fill Partitioned Tables for Scenario----------

CREATE TABLE Examples.InvoicesPart_1
(	InvoiceID int NOT NULL
		CONSTRAINT PK_Examples_InvoicesPart_1 PRIMARY KEY
		CHECK (InvoiceID BETWEEN 1 AND 20000),
	CustomerID int NOT NULL,
	InvoiceDate date NOT NULL,
	CustomerPurchaseOrderNumber nvarchar(20) NULL
);
GO

CREATE TABLE Examples.InvoicesPart_2
(	InvoiceID int NOT NULL
		CONSTRAINT PK_Examples_InvoicesPart_2 PRIMARY KEY
		CHECK (InvoiceID BETWEEN 20001 AND 40000),
	CustomerID int NOT NULL,
	InvoiceDate date NOT NULL,
	CustomerPurchaseOrderNumber nvarchar(20) NULL
);
GO

CREATE TABLE Examples.InvoicesPart_3
(	InvoiceID int NOT NULL
		CONSTRAINT PK_Examples_InvoicesPart_3 PRIMARY KEY
		CHECK (InvoiceID BETWEEN 40001 AND 60000),
	CustomerID int NOT NULL,
	InvoiceDate date NOT NULL,
	CustomerPurchaseOrderNumber nvarchar(20) NULL
);
GO

---------------------------------------------------------
--Insert data into the three partitioned tables---------
--------------------------------------------------------
INSERT INTO 
	Examples.InvoicesPart_1 (InvoiceID, CustomerID, InvoiceDate, CustomerPurchaseOrderNumber)
	SELECT InvoiceID, CustomerID, InvoiceDate, CustomerPurchaseOrderNumber
	FROM WideWorldImporters.Sales.Invoices
	WHERE InvoiceID BETWEEN 1 AND 20000;
GO

INSERT INTO 
	Examples.InvoicesPart_2 (InvoiceID, CustomerID, InvoiceDate, CustomerPurchaseOrderNumber)
	SELECT InvoiceID, CustomerID, InvoiceDate, CustomerPurchaseOrderNumber
	FROM WideWorldImporters.Sales.Invoices
	WHERE InvoiceID BETWEEN 20001 AND 40000;
GO

INSERT INTO 
	Examples.InvoicesPart_3 (InvoiceID, CustomerID, InvoiceDate, CustomerPurchaseOrderNumber)
	SELECT InvoiceID, CustomerID, InvoiceDate, CustomerPurchaseOrderNumber
	FROM WideWorldImporters.Sales.Invoices
	WHERE InvoiceID BETWEEN 40001 AND 60000;
GO

DROP VIEW IF EXISTS Examples.vPartitionedInvoices;
GO


--Create partitioned view using partitioned tables-------------------------
CREATE VIEW Examples.vPartitionedInvoices
AS
	SELECT InvoiceID, CustomerID, InvoiceDate, CustomerPurchaseOrderNumber
	FROM Examples.InvoicesPart_1
	UNION ALL
	SELECT InvoiceID, CustomerID, InvoiceDate, CustomerPurchaseOrderNumber
	FROM Examples.InvoicesPart_2
	UNION ALL
	SELECT InvoiceID, CustomerID, InvoiceDate, CustomerPurchaseOrderNumber
	FROM Examples.InvoicesPart_3;
GO

----------------------------------------------------------------
--Implementing the partitioned view-----------------------------
SELECT *
FROM Examples.vPartitionedInvoices
WHERE InvoiceID = 2756;

SELECT *
FROM Examples.vPartitionedInvoices
WHERE InvoiceID IN (18829, 52594);
GO

SELECT *
FROM Examples.vPartitionedInvoices
WHERE InvoiceDate = '2014-06-05'
GO

-----------------------------------------------------------
--Displaying the data in Sales.Invoices table for reference
--SELECT  [InvoiceID]
--      ,[CustomerID]
--      ,[InvoiceDate]
--      ,[CustomerPurchaseOrderNumber]
      
--  FROM [WideWorldImporters].[Sales].[Invoices]
--  ORDER BY CustomerID;

------------------------------------------------------------
--Clean up lab code if desired:
DROP VIEW IF EXISTS Examples.vPartitionedInvoices;
DROP TABLE IF EXISTS Examples.InvoicesPart_1;
DROP TABLE IF EXISTS Examples.InvoicesPart_2;
DROP TABLE IF EXISTS Examples.InvoicesPart_3;
GO

