/* 70-762 Lesson 6 Lab 3 - Creating and Calling Stored Procedures Using Table-valued Parameters */

USE WideWorldImporters;
GO

------------------------------------------------------------------------
--Set up lab by creating and filling two tables in the Examples schema---
-------------------------------------------------------------------------

DROP TABLE IF EXISTS Examples.LabCities;
GO

DROP TABLE IF EXISTS Examples.LabStateProvinces;
GO

SELECT CityID, CityName, StateProvinceID, LatestRecordedPopulation, ValidFrom 
INTO Examples.LabCities
FROM Application.Cities;
GO

SELECT StateProvinceID, StateProvinceName, CountryID, LatestRecordedPopulation, ValidFrom
INTO Examples.LabStateProvinces
FROM Application.StateProvinces;
GO

SELECT *
FROM.Examples.LabCities;
GO

SELECT *
FROM Examples.LabStateProvinces;
GO


--Create table type and define table structure
--Create procedure that has a parameter of the table type
--Declare a variable of table type, and reference table type
--Fill the table variable by using an INSERT statement
--After the table variable is created and filled, pass table variable data to stored procedure

DROP TYPE IF EXISTS Examples.LabCitiesTableType;
GO

/* Create a table type. */  
CREATE TYPE Examples.LabCitiesTableType AS TABLE   
( CityID INT
, CityName NVARCHAR(50)
, StateProvinceID INT  
, LatestRecordedPopulation BIGINT );  
GO  

DROP PROC IF EXISTS Examples.uspInsertExamplesLabCities;
GO

/* Create a procedure to receive data for the table-valued parameter. */  
CREATE PROCEDURE Examples.uspInsertExamplesLabCities  
    @TVP Examples.LabCitiesTableType READONLY  
    AS   
    SET NOCOUNT ON  
    INSERT INTO WideWorldImporters.Examples.LabCities  
           (CityID
		   ,CityName  
           ,StateProvinceID  
           ,LatestRecordedPopulation  
           ,ValidFrom)  
        SELECT *, GETDATE()  
        FROM  @TVP;  
        GO  

-------------------
/* Declare a variable that references the type. */  
DECLARE @LabCitiesTVP AS Examples.LabCitiesTableType;  
/* Add data to the table variable. */  
INSERT INTO @LabCitiesTVP (CityID, CityName, StateProvinceID, LatestRecordedPopulation)  
    SELECT StateProvinceID, StateProvinceName,CountryID,LatestRecordedPopulation  
    FROM WideWorldImporters.Examples.LabStateProvinces;  
/* Pass the table variable data to a stored procedure. */  
EXEC Examples.uspInsertExamplesLabCities  @LabCitiesTVP;  
GO
-------------------

--Examples.LabCities now includes the data (53 stateprovinces) from the table parameter of the stored procedure
SELECT *
FROM.Examples.LabCities;
GO

--Examples LabStateProvinces is not changed
SELECT *
FROM Examples.LabStateProvinces;
GO


