/* 70-762 Lesson 6 Lab 4 - Creating and Executing Stored Procedures with RETURN Statements */

USE WideWorldImporters;
GO

DROP PROC IF EXISTS Sales.uspCheckCustomerCity;
GO

--The following example shows the return status from executing uspCheckCustomerCity
CREATE PROCEDURE Sales.uspCheckCustomerCity
   (@cityName CHAR(50))
AS
BEGIN
   IF ((SELECT COUNT(*)
   FROM Sales.Customers
   WHERE PostalAddressLine2 = @cityName) > 1)
   RETURN 1
ELSE
   RETURN 0
END;
GO

--The @return_status local variable must be declared before it can be used
--If the input city is in the Customers table, return_status returns 1, otherwise 0
DECLARE @return_status int;  
EXEC @return_status = Sales.uspCheckCustomerCity 'Almeidaville';  
SELECT 'Return Status' = @return_status;  
GO

---------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------
--EXTRA--------------------------------------------------------------------------------
USE AdventureWorks2016CTP3;
GO

DROP PROC IF EXISTS Sales.uspGetYTDSales
GO

---------------------------------------------------
CREATE PROCEDURE Sales.uspGetYTDSales
@SalesPerson nvarchar(50) = NULL,  -- NULL default value
@SalesYTD money = NULL OUTPUT
AS  
-- Validate the @SalesPerson parameter.
IF @SalesPerson IS NULL
   BEGIN
       PRINT 'ERROR: Please specify a last name for the sales person'
       RETURN(1)
   END
ELSE
   BEGIN
   -- Determine if the specified parameter value (last name) is valid
   IF (SELECT COUNT(*) FROM HumanResources.vEmployee
          WHERE LastName = @SalesPerson) = 0
      RETURN(2)
   END
-- Get the sales for the specified name and 
-- assign it to the output parameter
SELECT @SalesYTD = SalesYTD 
FROM Sales.SalesPerson AS sp
JOIN HumanResources.vEmployee AS e ON e.BusinessEntityID = sp.BusinessEntityID
WHERE LastName = @SalesPerson;
-- Check for SQL Server errors
IF @@ERROR <> 0 
   BEGIN
      RETURN(3)
   END
ELSE
   BEGIN
   -- Check to see if the specified name has any year to date sales
     IF @SalesYTD IS NULL
       RETURN(4) 
     ELSE
      -- Successful execution of proc
        RETURN(0)
   END;
   GO
-------------------------------------------------------------------------------

-- Run the stored procedure without specifying an input value.
EXEC Sales.uspGetYTDSales;
GO

--*****Saving OUTPUT value with a variable**********************************
-- Run the stored procedure with an input value.
DECLARE @SalesYTDForSalesPerson money, @ret_code int;
-- Execute the procedure specifying a last name for the input parameter
-- and saving the output value in the variable @SalesYTD
EXECUTE Sales.uspGetYTDSales
    N'Blythe', @SalesYTD = @SalesYTDForSalesPerson OUTPUT;
PRINT N'Year-to-date sales for this employee is ' +
    CONVERT(varchar(10), @SalesYTDForSalesPerson);
--Also try Campbell---

--******USING RETURN CODES*************************************************
--Handling the different return codes that are returned from a stored procedure
--The following creates a program to handle the return codes that are returned 
--from the Sales.uspGetYTDSales procedure

-- Declare the variables to receive the output value and return code 
-- of the procedure.
DECLARE @SalesYTDForSalesPerson money, @ret_code int;
-- Execute the procedure with a title_id value
-- and save the output value and return code in variables.
EXECUTE @ret_code = Sales.uspGetYTDSales
    N'Blythe', @SalesYTD = @SalesYTDForSalesPerson OUTPUT;
--  Check the return codes.
IF @ret_code = 0
BEGIN
   PRINT 'Procedure executed successfully'
   -- Display the value returned by the procedure.
   PRINT 'Year-to-date sales for this employee is ' + CONVERT(varchar(10),@SalesYTDForSalesPerson)
END
ELSE IF @ret_code = 1
   PRINT 'ERROR: Please specify a last name for the sales person'
ELSE IF @ret_code = 2 
   PRINT 'ERROR: Please enter a valid last name for the sales person'
ELSE IF @ret_code = 3
   PRINT 'ERROR: An error occurred getting sales value'
ELSE IF @ret_code = 4
   PRINT 'ERROR: No sales recorded for this employee'   
GO
 
--------------------------------------------------------------------------------
---EXECUTE PROC USING ANOTHER EMPLOYEE-----------------------------------------
-- Declare the variables to receive the output value and return code 
-- of the procedure.
DECLARE @SalesYTDForSalesPerson money, @ret_code int;
-- Execute the procedure with a title_id value
-- and save the output value and return code in variables.
EXECUTE @ret_code = Sales.uspGetYTDSales
    N'Welcker', @SalesYTD = @SalesYTDForSalesPerson OUTPUT;
--  Check the return codes.
IF @ret_code = 0
BEGIN
   PRINT 'Procedure executed successfully'
   -- Display the value returned by the procedure.
   PRINT 'Year-to-date sales for this employee is ' + CONVERT(varchar(10),@SalesYTDForSalesPerson)
END
ELSE IF @ret_code = 1
   PRINT 'ERROR: Please specify a last name for the sales person'
ELSE IF @ret_code = 2 
   PRINT 'ERROR: Please enter a valid last name for the sales person'
ELSE IF @ret_code = 3
   PRINT 'ERROR: An error occurred getting sales value'
ELSE IF @ret_code = 4
   PRINT 'ERROR: No sales recorded for this employee'   
GO
