/* 70-762 Lesson 8 Lab 2 - Implementing Transaction Control */


USE [WideWorldImporters]
GO

--Set up tables for lab------------------------------
DROP TABLE IF EXISTS Examples.CustTran;
GO

DROP TABLE IF EXISTS Examples.CustCatTran;
GO

SELECT CustomerCategoryID
      ,CustomerCategoryName
	INTO Examples.CustCatTran
FROM Sales.CustomerCategories
WHERE CustomerCategoryID IN (3, 4, 5, 6);
GO

ALTER TABLE Examples.CustCatTran
ADD CONSTRAINT PK_CustCatTran_CustomerCategoryID PRIMARY KEY CLUSTERED (CustomerCategoryID);
GO


SELECT CustomerID
      ,CustomerCategoryID
	  ,CustomerName
	 INTO Examples.CustTran 
FROM Sales.Customers
WHERE CustomerID BETWEEN 905 AND 910;
GO

ALTER TABLE Examples.CustTran
ADD CONSTRAINT PK_CustTran_CustomerID PRIMARY KEY CLUSTERED (CustomerID);
GO

ALTER TABLE Examples.CustTran
ADD CONSTRAINT FK_CustTran_Ref_CustCatTran
	FOREIGN KEY (CustomerCategoryID) REFERENCES Examples.CustCatTran(CustomerCategoryID);
GO

SELECT * FROM Examples.CustCatTran;
SELECT * FROM Examples.CustTran;

---Example 1 Multiple DML statements in Transaction--------------
BEGIN TRANSACTION;
	UPDATE Examples.CustCatTran
	SET CustomerCategoryName = 'Novelty Toy Shop'
	WHERE CustomerCategoryName = 'Novelty Shop';

	UPDATE Examples.CustTran
	SET CustomerName = 'Minnie Mouse'
	WHERE CustomerName = 'Sumati Bhat';
COMMIT TRANSACTION;

SELECT CustCatTran.CustomerCategoryID, CustomerCategoryName, CustomerID, CustomerName
FROM Examples.CustCatTran
	FULL OUTER JOIN
	Examples.CustTran
ON CustCatTran.CustomerCategoryID = CustTran.CustomerCategoryID;
GO

-------------------------------------------------
-------------------------------------------------
--Example 2--------------------------------------
--Multiple DML statements in Transaction---------
--Insert statement commits but DELETE statement fails
BEGIN TRANSACTION;
	INSERT INTO 
	Examples.CustCatTran(CustomerCategoryID
                       ,CustomerCategoryName)
	VALUES (10, 'Boutique');

	DELETE 
	Examples.CustCatTran
	WHERE CustomerCategoryName = 'Supermarket';
COMMIT TRANSACTION;

----------------------------------------
SELECT CustCatTran.CustomerCategoryID, CustomerCategoryName, CustomerID, CustomerName
FROM Examples.CustCatTran
	FULL OUTER JOIN
	Examples.CustTran
ON CustCatTran.CustomerCategoryID = CustTran.CustomerCategoryID;
GO

--Example 3--------------------------------------
--//////////Setting XACT_ABORT ON///////////// 
--Multiple DML statements in Transaction---------
--DELETE statment fails------
--Since XACT_ABORT ON entire transaction rolled back-------
SET XACT_ABORT ON;
GO

BEGIN TRANSACTION;
	INSERT INTO 
	Examples.CustCatTran(CustomerCategoryID
                       ,CustomerCategoryName)
	VALUES (11, 'Pet Store');

	DELETE 
	Examples.CustCatTran
	WHERE CustomerCategoryName = 'Supermarket';
COMMIT TRANSACTION;

SET XACT_ABORT OFF;
GO

-----------------------------------------------
SELECT CustCatTran.CustomerCategoryID, CustomerCategoryName, CustomerID, CustomerName
FROM Examples.CustCatTran
	FULL OUTER JOIN
	Examples.CustTran
ON CustCatTran.CustomerCategoryID = CustTran.CustomerCategoryID;
GO

--Example 4----------------------------------
--Use TRY CATCH Block to ROLLBACK Transaction when it fails
BEGIN TRY
	BEGIN TRANSACTION;
		INSERT INTO 
		Examples.CustCatTran(CustomerCategoryID
                       ,CustomerCategoryName)
		VALUES (11, 'Pet Store');

		DELETE 
		Examples.CustCatTran
		WHERE CustomerCategoryName = 'Supermarket';
	COMMIT TRANSACTION;
END TRY
BEGIN CATCH
	IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION;
END CATCH;

-----------------------------------------------
SELECT CustCatTran.CustomerCategoryID, CustomerCategoryName, CustomerID, CustomerName
FROM Examples.CustCatTran
	FULL OUTER JOIN
	Examples.CustTran
ON CustCatTran.CustomerCategoryID = CustTran.CustomerCategoryID;
GO

--Example 5------------------------------------
--/////////////////////////////////////////////
----------------------------------------------
/*>>>>Now Test IMPLICIT TRANSACTIONS<<<<<<<<<*/
SET IMPLICIT_TRANSACTIONS ON;
GO

--NOTICE NO BEGIN TRANSACTION-----------------
INSERT INTO 
	Examples.CustCatTran(CustomerCategoryID
                       ,CustomerCategoryName)
	VALUES (11, 'Pet Store');
SELECT @@TRANCOUNT AS 'TRANCOUNT';
DELETE 
	Examples.CustCatTran
	WHERE CustomerCategoryName = 'Supermarket';
COMMIT TRANSACTION;

SELECT @@TRANCOUNT AS 'TRANCOUNT';

----------------
SELECT CustCatTran.CustomerCategoryID, CustomerCategoryName, CustomerID, CustomerName
FROM Examples.CustCatTran
	FULL OUTER JOIN
	Examples.CustTran
ON CustCatTran.CustomerCategoryID = CustTran.CustomerCategoryID;
GO

DECLARE @IMPLICIT_TRANSACTIONS VARCHAR(3) = 'OFF';
  IF ( (2 & @@OPTIONS) = 2 ) SET @IMPLICIT_TRANSACTIONS = 'ON';
SELECT @IMPLICIT_TRANSACTIONS AS IMPLICIT_TRANSACTIONS;
GO

SET IMPLICIT_TRANSACTIONS OFF;

DECLARE @IMPLICIT_TRANSACTIONS VARCHAR(3) = 'OFF';
  IF ( (2 & @@OPTIONS) = 2 ) SET @IMPLICIT_TRANSACTIONS = 'ON';
SELECT @IMPLICIT_TRANSACTIONS AS IMPLICIT_TRANSACTIONS;
GO

DROP TABLE IF EXISTS Examples.CustTran;
GO

DROP TABLE IF EXISTS Examples.CustCatTran;
GO


