/* 70-762 Lesson 8 Lab 3 - Implementing SAVE TRANSACTION and ROLLBACK TRANSACTION */

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
--TRANSACTION INSIDE STORED PROCEDURE USING SAVEPOINT-----------
-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
-- This stored procedure will roll back a transaction to a savepoint if 
-- the @ROLLBK parameter is a 1

USE WideWorldImporters;
GO

DROP TABLE IF EXISTS Examples.TranExampleTable;
GO

DROP PROCEDURE IF EXISTS Examples.uspSavepointExample;
GO

CREATE TABLE Examples.TranExampleTable (value int)
GO

SET XACT_ABORT, NOCOUNT ON;
GO

CREATE PROCEDURE Examples.uspSavepointExample @Value int, @RollBk bit
AS
BEGIN
BEGIN TRANSACTION
SAVE TRANSACTION TranSavept --savepoint created
INSERT INTO Examples.TranExampleTable VALUES (@Value)
	PRINT @@TRANCOUNT;
IF @Rollbk = 1 
-- Roll back to the savepoint
-- The transaction is not closed and 
-- @@TRANCOUNT is not changed.
ROLLBACK TRANSACTION TranSavept --transaction rolls back to savepoint
-- Close the transaction created at the beginning of the procedure
COMMIT
END
GO

--Delete values from the table 
TRUNCATE TABLE Examples.TranExampleTable;
GO

-- Begin a new transaction -  testing to understand how the savepoint works
PRINT @@TRANCOUNT;
BEGIN TRANSACTION
PRINT @@TRANCOUNT;
INSERT INTO Examples.TranExampleTable VALUES (7) --this value of 7 is retained in the table, inserted before savepoint
-- Run the stored procedure with the param to roll back a transaction and trying to insert 5 into the table
PRINT @@TRANCOUNT;
EXEC Examples.uspSavepointExample 5,1
PRINT @@TRANCOUNT;
-- Run the commit to close the initial transaction
COMMIT
PRINT @@TRANCOUNT;
SELECT * FROM Examples.TranExampleTable;
PRINT @@TRANCOUNT;
GO

------------------------------------------------------------------
------------------------------------------------------------------
--Alter the procedure by removing (commenting out) the savepoint
--Run the same example-------------------------------------------
ALTER PROCEDURE Examples.uspSavepointExample @Value int, @RollBk bit
AS
BEGIN
BEGIN TRANSACTION
--SAVE TRANSACTION TranSavept --savepoint created (commented out)
INSERT INTO Examples.TranExampleTable VALUES (@Value)
	PRINT @@TRANCOUNT;
IF @Rollbk = 1 
-- Roll back to the savepoint
-- The transaction is not closed and 
-- @@TRANCOUNT is not changed.
ROLLBACK TRANSACTION --TranSavept --transaction rolls back to savepoint (commented out)
-- Close the transaction created at the beginning of the procedure
COMMIT
END
GO

--Delete values from the table 
TRUNCATE TABLE Examples.TranExampleTable;
GO

-- Begin a new transaction testing to understand the steps without the savepoint
PRINT @@TRANCOUNT;
BEGIN TRANSACTION
PRINT @@TRANCOUNT;
INSERT INTO Examples.TranExampleTable VALUES (7) --this value of 7 is inserted before the transaction errors out
-- Run the stored procedure with the param to roll back a transaction and trying to insert 5 into the table
PRINT @@TRANCOUNT;
EXEC Examples.uspSavepointExample 5,1
PRINT @@TRANCOUNT;
-- Run the commit to close the initial transaction
COMMIT
PRINT @@TRANCOUNT;
SELECT * FROM Examples.TranExampleTable; -- note no values in table - everything rolled back
PRINT @@TRANCOUNT;
GO
 
 SET XACT_ABORT OFF;
 GO

--Cleanup by dropping table and procedure
DROP TABLE IF EXISTS Examples.TranExampleTable;
GO

DROP PROCEDURE IF EXISTS Examples.uspSavepointExample;
GO
